
#include "common.h"
#include "Sprite.h"
#include "TouchPanel.h"
#include "Sound.h"


// スプライト
enum
{
	SPR_PHOTO = 0,			// 背景
	SPR_MAX,
};

static sys::Sprite*	sprite;		// スプライト


// サウンドデータ
enum
{
	SOUND_BGM = 0,			// BGM
	SOUND_SE,				// SE
	SOUND_MAX,
};

static sys::SoundPlayer*	sound_player;				// サウンドプレイヤー
static char*				sound_data[SOUND_MAX];		// サウンドデータ
static u32					sound_size[SOUND_MAX];		// サウンドデータサイズ
static int					se_track;					// SEトラック番号


/************
    初期化
 ************/
void	init_app(void)
{
	struct SprDef
	{
		const char*		tex_name;		// テクスチャファイル名
		SRect			coord;			// UV座標
	};

	static const
	SprDef	spr_def[SPR_MAX] =
			{
				{"photo.pkm",		{  0,   0, 640, 960}},		// 背景
			};

	sprite = new sys::Sprite[SPR_MAX];				// スプライト初期化
	for (int i = 0; i < SPR_MAX; i++) {
		sprite[i].set(sys::TexCache::RES_ASSET, spr_def[i].tex_name, &spr_def[i].coord);
	}


	sound_player = new sys::SoundPlayer[5];													// サウンドプレイヤー
	sound_data[SOUND_BGM] = (char*)sys::load_asset("bgm.wav", &sound_size[SOUND_BGM]);		// サウンドデータ読み込み
	sound_data[SOUND_SE] = (char*)sys::load_asset("se.wav", &sound_size[SOUND_SE]);
	se_track = 0;

	sound_player[4].play(sound_data[SOUND_BGM], sound_size[SOUND_BGM], 0);					// BGM再生開始
}

/**********
    終了
 **********/
void	quit_app(void)
{
	delete[]	sound_player;
	for (int i = 0; i < SOUND_MAX; i++) {
		free(sound_data[i]);
	}

	delete[]	sprite;
}

/******************************
    稼働
		戻り値	アプリ続行か
 ******************************/
Bool	update_app(void)
{
	sprite[SPR_PHOTO].draw(0.0f, 0.0f);			// 背景

	if ( sys::TouchPanel[0].flag & sys::TouchManager::TRIGGER ) {
		sound_player[se_track].play(sound_data[SOUND_SE], sound_size[SOUND_SE]);			// SE再生
		se_track = ++se_track % 4;
	}

	return	TRUE;
}

/**************
    一時停止
 **************/
void	pause_app(void)
{
	for (int i = 0; i < 5; i++) {
		sound_player[i].pause();
	}
}

/**********
    再開
 **********/
void	resume_app(void)
{
	for (int i = 0; i < 5; i++) {
		sound_player[i].resume();
	}
}

/**************** End of File *************************************************/