/***************************

		サウンド管理

 ***************************/

#include "Sound.h"


namespace sys
{

SLObjectItf		SoundPlayer::engineObject = NULL;		// エンジンオブジェクト
SLEngineItf		SoundPlayer::engineEngine;				// インタフェース
float			SoundPlayer::master_volume;				// マスター音量

/********************
    エンジン初期化
 ********************/
void	SoundPlayer::init_engine(void)
{
	SLresult	result;

	result = slCreateEngine(&engineObject, 0, NULL, 0, NULL, NULL);			// エンジンオブジェクト作成
	assert(SL_RESULT_SUCCESS == result);

	result = (*engineObject)->Realize(engineObject, SL_BOOLEAN_FALSE);		// リアライズ
	assert(SL_RESULT_SUCCESS == result);

	result = (*engineObject)->GetInterface(engineObject, SL_IID_ENGINE, &engineEngine);
	assert(SL_RESULT_SUCCESS == result);									// インタフェース取得

	master_volume = 1.0f;				// マスター音量初期化
}

/******************
    エンジン終了
 ******************/
void	SoundPlayer::quit_engine(void)
{
	if ( engineObject ) {
		(*engineObject)->Destroy(engineObject);
		engineObject = NULL;
	}
}


/********************
    コンストラクタ
 ********************/
SoundPlayer::SoundPlayer(void)
{
	SLresult	result;

	result = (*engineEngine)->CreateOutputMix(engineEngine, &outputMixObject, 0, NULL, NULL);
	assert(SL_RESULT_SUCCESS == result);									// 出力オブジェクト作成

	result = (*outputMixObject)->Realize(outputMixObject, SL_BOOLEAN_FALSE);
	assert(SL_RESULT_SUCCESS == result);									// リアライズ

	bqPlayerObject = NULL;

	state = STOPPED;
}

/******************
    デストラクタ
 ******************/
SoundPlayer::~SoundPlayer()
{
	stop();
	(*outputMixObject)->Destroy(outputMixObject);
}


/**********************
    再生コールバック
 **********************/
static
void	bqPlayerCallbackWAV(SLAndroidSimpleBufferQueueItf, void* context)
{
	((SoundPlayer*)context)->callback_wav();
}

void	SoundPlayer::callback_wav(void)
{
	if ( sound_loop == 1 ) {
		(*bqPlayerPlay)->SetPlayState(bqPlayerPlay, SL_PLAYSTATE_STOPPED);						// 停止
		state = STOPPED;
		return;
	}
	if ( sound_loop > 1 ) {
		sound_loop--;
	}
	(*bqPlayerBufferQueue)->Enqueue(bqPlayerBufferQueue, sound_data, (SLuint32)sound_size);		// 再生開始
}


struct WaveFormat
{
	s8		riff[4];
	u32		total_size;
	s8		fmt[8];
 	u32		fmt_size;
	u16		format;
	u16		channel;
	u32		rate;
	u32		avgbyte;
	u16		block;
	u16		bit;
	s8		data[4];
	u32		data_size;
};

/****************************************************
    再生
		引数	_data = サウンドデータ
				_size = サウンドデータサイズ
				_loop = ループ回数（0:無限ループ）
				_vol  = 音量
 ****************************************************/
void	SoundPlayer::prepare(const void* _data, u32 _size, int _loop, float _vol)
{
	stop();


	SLDataLocator_AndroidSimpleBufferQueue	loc_bufq = {SL_DATALOCATOR_ANDROIDSIMPLEBUFFERQUEUE, 2};
	SLDataFormat_PCM						format_pcm;
	SLDataSource							audioSrc = {&loc_bufq, &format_pcm};

	switch ( *((u32*)_data) ) {						// データフォーマット
	  case 0x46464952 :			// WAV
		{
			WaveFormat*		_info = (WaveFormat*)_data;

			format_pcm.formatType		= SL_DATAFORMAT_PCM;
			format_pcm.numChannels		= (SLuint32)_info->channel;
			format_pcm.samplesPerSec	= (SLuint32)_info->rate*1000;
			format_pcm.bitsPerSample	= (SLuint32)_info->bit;
			format_pcm.containerSize	= (SLuint32)_info->bit;
			format_pcm.channelMask		= (_info->channel == 1) ? SL_SPEAKER_FRONT_CENTER : (SL_SPEAKER_FRONT_LEFT | SL_SPEAKER_FRONT_RIGHT);
			format_pcm.endianness		= SL_BYTEORDER_LITTLEENDIAN;

			sound_data = (char*)((u32)_data + sizeof(WaveFormat));
			sound_size = _info->data_size;
			sound_loop = _loop;
		}
		break;

	  default :
		assert(FALSE);
		break;
	}

	SLDataLocator_OutputMix		loc_outmix = {SL_DATALOCATOR_OUTPUTMIX, outputMixObject};
	SLDataSink					audioSnk = {&loc_outmix, NULL};

	const SLInterfaceID		ids[3] = {SL_IID_PLAY, SL_IID_BUFFERQUEUE, SL_IID_VOLUME};
	const SLboolean			req[3] = {SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE};

	SLresult	result;

	result = (*engineEngine)->CreateAudioPlayer(engineEngine, &bqPlayerObject, &audioSrc, &audioSnk, 3, ids, req);
	if ( SL_RESULT_SUCCESS != result ) {									// プレイヤーオブジェクト作成
		bqPlayerObject = NULL;
		return;
	}

	result = (*bqPlayerObject)->Realize(bqPlayerObject, SL_BOOLEAN_FALSE);	// リアライズ
	assert(SL_RESULT_SUCCESS == result);

	result = (*bqPlayerObject)->GetInterface(bqPlayerObject, SL_IID_PLAY, &bqPlayerPlay);
	assert(SL_RESULT_SUCCESS == result);									// インタフェース取得

	result = (*bqPlayerObject)->GetInterface(bqPlayerObject, SL_IID_BUFFERQUEUE, &bqPlayerBufferQueue);
	assert(SL_RESULT_SUCCESS == result);									// バッファキューインタフェース

	result = (*bqPlayerObject)->GetInterface(bqPlayerObject, SL_IID_VOLUME, &bqPlayerVolume);
	assert(SL_RESULT_SUCCESS == result);									// 音量インタフェース
	set_volume(_vol);														// 音量設定

	result = (*bqPlayerBufferQueue)->RegisterCallback(bqPlayerBufferQueue, bqPlayerCallbackWAV, this);
	assert(SL_RESULT_SUCCESS == result);									// 再生コールバック設定

	(*bqPlayerBufferQueue)->Enqueue(bqPlayerBufferQueue, sound_data, (SLuint32)sound_size);

	state = PREPARED;
}

void	SoundPlayer::play(void)
{
	if ( bqPlayerObject ) {
		state = PLAYING;
		(*bqPlayerPlay)->SetPlayState(bqPlayerPlay, SL_PLAYSTATE_PLAYING);			// 再生開始
	}
}

void	SoundPlayer::play(const void* _data, u32 _size, int _loop, float _vol)
{
	prepare(_data, _size, _loop, _vol);
	play();
}


/**********
    停止
 **********/
void	SoundPlayer::stop(void)
{
	if ( bqPlayerObject ) {					// 再生中
		(*bqPlayerPlay)->SetPlayState(bqPlayerPlay, SL_PLAYSTATE_STOPPED);			// 停止状態
		(*bqPlayerObject)->Destroy(bqPlayerObject);
		bqPlayerObject = NULL;
	}
	state = STOPPED;
}


/*****************************************************
    音量設定
		引数	_vol = 音量（0.0:最小 ～ 1.0:最大）
 *****************************************************/
void	SoundPlayer::set_volume(float _vol)
{
	volume = _vol;
	set_volume();
}

void	SoundPlayer::set_volume(void)
{
	if ( bqPlayerObject ) {
		float		_vol = volume*master_volume;
		SLresult	result;

		result = (*bqPlayerVolume)->SetVolumeLevel(bqPlayerVolume, (_vol >= 1.0f) ? 0 : ((_vol < 0.01f) ? -16000
																							: (SLmillibel)(8000.0f*log10f(_vol))));
		assert(SL_RESULT_SUCCESS == result);
	}
}

/*************************************************************
    マスター音量設定
		引数	_vol = マスター音量（0.0:最小 ～ 1.0:最大）
		戻り値	マスター音量が変更されたか
 *************************************************************/
Bool	SoundPlayer::set_master_volume(float _vol)
{
	if ( master_volume != _vol ) {
		master_volume = _vol;
		return	TRUE;
	}
	return	FALSE;
}


/**************
    一時停止
 **************/
void	SoundPlayer::pause(void)
{
	if ( state == PLAYING ) {				// 再生中
		if ( bqPlayerObject ) {
			(*bqPlayerPlay)->SetPlayState(bqPlayerPlay, SL_PLAYSTATE_PAUSED);		// 一時停止状態
		}
		state = PAUSED;
	}
}

/**********
    再開
 **********/
void	SoundPlayer::resume(void)
{
	if ( state == PAUSED ) {				// 一時停止中
		play();
	}
}

}

/**************** End of File ******************************************/
