
#include "common.h"
#include "Renderer.h"
#include "TouchPanel.h"
#include "Sound.h"


using namespace sys;


extern "C"
{
JNIEXPORT void JNICALL		Java_sys_BaseActivity_createNative(JNIEnv*, jobject);
JNIEXPORT int JNICALL		Java_sys_BaseActivity_initNative(JNIEnv*, jobject, jint, jint, jobject);
JNIEXPORT void JNICALL		Java_sys_BaseActivity_quitNative(JNIEnv*, jobject);
JNIEXPORT void JNICALL		Java_sys_BaseActivity_pauseNative(JNIEnv*, jobject);
JNIEXPORT jboolean JNICALL	Java_sys_BaseActivity_updateNative(JNIEnv*, jobject, jbyteArray);
}

void	init_app(void);			// メイン初期化
void	quit_app(void);			// メイン終了
void	pause_app(void);		// メイン一時停止
void	resume_app(void);		// メイン再開
Bool	update_app(void);		// メイン稼働

static Bool		init_flag = FALSE;				// 初期化フラグ


/**********
    開始
 **********/
JNIEXPORT void JNICALL	Java_sys_BaseActivity_createNative(JNIEnv*, jobject)
{
	init_flag = FALSE;			// 初期化フラグ
}

/************
    初期化
 ************/
JNIEXPORT int JNICALL	Java_sys_BaseActivity_initNative(JNIEnv* env, jobject, jint width, jint height, jobject mgr)
{
	LOGI("initNative (%d, %d)", width, height);

	asset_manager = AAssetManager_fromJava(env, mgr);		// asset読み込みマネージャー
	assert(asset_manager != NULL);

	Renderer::init(width, height);				// 描画管理初期化
	TouchManager::init_manager();				// タッチパネル管理初期化
	SoundPlayer::init_engine();					// サウンド初期化

	if ( !init_flag ) {
		init_app();								// アプリメイン初期化
		init_flag = TRUE;
	}
	else {
		resume_app();							// アプリメイン再開
	}

	return	FRAME_PERIOD;
}

/**********
    終了
 **********/
JNIEXPORT void JNICALL	Java_sys_BaseActivity_quitNative(JNIEnv*, jobject)
{
	LOGI("quitNative");

	quit_app();									// アプリメイン終了
	SoundPlayer::quit_engine();					// サウンド終了
	init_flag = FALSE;
}

/**************
    一時停止
 **************/
JNIEXPORT void JNICALL	Java_sys_BaseActivity_pauseNative(JNIEnv*, jobject)
{
	LOGI("pauseNative");

	if ( init_flag ) {
		pause_app();							// アプリメイン一時停止
	}
	TouchManager::quit_manager();				// タッチパネル管理終了
	Renderer::quit();							// 描画管理終了
}

/**********
    稼働
 **********/
JNIEXPORT jboolean JNICALL	Java_sys_BaseActivity_updateNative(JNIEnv* env, jobject, jbyteArray touch)
{
	Renderer::update();							// 描画管理稼働

	jbyte*	dst = (jbyte*)env->GetPrimitiveArrayCritical(touch, NULL);

	TouchManager::update_manager((short*)dst);	// タッチパネル管理稼働
	env->ReleasePrimitiveArrayCritical(touch, dst, 0);

	if ( update_app() ) {						// アプリメイン稼働
		Renderer::draw();						// 描画後処理
		return	JNI_TRUE;
	}
	init_flag = FALSE;
	return	JNI_FALSE;							// アプリ終了
}


namespace sys
{

AAssetManager*	asset_manager;			// asset読み込み用

/****************************************
    assetファイル読み込み
			引数	name = ファイル名
			戻り値	データ
					size：データサイズ
 ****************************************/
void*	load_asset(const char* name, u32* size)
{
	AAsset*	_asset = AAssetManager_open(asset_manager, name, AASSET_MODE_BUFFER);

	assert(_asset);

	size_t	_size = AAsset_getLength(_asset);		// ファイルサイズ
	void*	_buf = malloc(_size);					// データバッファ

	AAsset_read(_asset, _buf, _size);				// データ読み込み
	AAsset_close(_asset);
	if ( size ) {
		*size = (u32)_size;
	}
	return	_buf;
}

}

/**************** End of File *************************************************/